body {
    background-color: white;
    color: black;
    font-size 12pt;
    height: auto;
    font-family: Cantarell, sans-serif;
    width: 100%;
    overflow-x: hidden !important;
}

a, a:visited, a:active, a:link {
    color: #3C6EB4;
    text-decoration: none;
}

legend {
    font-size: 1.2em;
    padding-bottom: 10px;
}

.pretty-list {
    list-style: none;
    margin-left: -25px;
}

.pretty-list li {
    display: table-cell;
    vertical-align: middle;
    background: url(../img/badge_bullet.png) no-repeat left center;
    padding-left: 50px;
    height: 40px;
}

.page {
    /* Without the below line, the page div is skewed
     * right. */
    margin-right: 15px;
}

.date {
    font-size: 0.8em;
    color: #9B9B9B; /* Dark gray */
    display: block;
    padding-top: 10px;
}

.navbar {
    list-style-type: none;
    background: #3158AF;
    color: white;
    width: 100%;
    padding: 5px;
    margin-left: -8px; /* There has to be a better way. */
    margin-top: 20px;
}

.navbar li {
    display: inline;
    padding: 10px;
}

.navbar a {
    color: white;
    text-decoration: none;
}

table {
    border-spacing: 0px;
    width: 100%;
}

td {
    padding: 10px;
}

tr:nth-child(odd) { background: #E8EFF8; } /* "Clear day" */

tr:nth-child(even) { background: white; }

.logo-image {
    margin-right: 10px;
    margin-top: -10px;
}

#logo {
    color: #9B9B9B; /* Dark gray */
}

#sublogo {
    vertical-align: 35%;
    padding-left: 15px;
    font-size: 50%;
}

/* This is applied to the content of columns, but not to the header,
 * so that the content is a bit less-wide than the header.
 * (on suggestion of Jenn Kotler)
 */
.padded-content {
    padding-left: 10px;
    padding-right: 10px;
}

#header-link {
    text-decoration: none;
}

.header {
    color: #3158AF;
    height: 50px;
    margin-top: -10px;
    margin-bottom: 20px;
}

.downshift {
    margin-top: 0px !important;
    margin-bottom: 0px !important;
}

.downshift-20 {
    margin-top: 20px !important;
    margin-bottom: 0px !important;
}

.section-header {
    background: #3158AF;
    color: white;
    text-align: center;
    padding: 10px;
}

#footer {
    color: #9B9B9B; /* Dark gray */
    font-size: 10px;
    text-align: center;
} 

.collapsed-row {
    visibility: collapse;
}

.badge-name {
    width: 50%;
}

.badge-image {
    width: 15%;
    text-align: center;
}

.badge-assertions {
    width: 35%;
}

.badge-assertions ul {
    list-style-type: none;
    margin: 0px;
    padding: 0px;
}

.badge-assertions li {
    padding-left: 15px;
}

li.popup {
    width: 150px;
}

div.popup {
    position: absolute;
    float: left;
    border: 1px solid black;
    background-color: #1E1E1E;
    background-color: rgba(30,30,30,0.75);
}

.weak {
    color: #666;
    font-style: italic;
}

.sublabel {
    display: block;
    font-size: 0.8em;
    color: #666;
}

/* Applied to every form element on the admin view. */
form {
    padding: 4px;
    margin-bottom: 20px;
}

.thumbnail-96 {
    width: 96px;
    height: 96px;
}

.thumbnail-64 {
    width: 64px;
    height: 64px;
}

.thumbnail {
    margin-left: auto;
    margin-right: auto;
}

.thumbnail-container {
    margin-top: 5px;
    margin-bottom: 5px;
}

.text-64 {
    text-align: center;
    margin-top: 18px;
}

.tooltip {
    /* Nothing, Lebowski. */
}

.tooltip::after {
    background: rgba(0, 0, 0, 0.8);
    border-radius: 8px 8px 8px 0px;
    box-shadow: 1px 1px 10px rgba(0, 0, 0, 0.5);
    color: #FFF;
    content: attr(data-tooltip); /* amazing */
    opacity: 0; /* Our element is transparent... */
    padding: 3px 7px;
    position: absolute;
    visibility: hidden; /* ...and hidden. */
    transition: all 0.4s ease-in-out; /* To add some smoothness */
}

.tooltip:hover::after {
    opacity: 1; /* Make it visible */
    visibility: visible;
}

.tooltip-64:after {
    margin-top: -96px;
    margin-left: 64px;
}

.tooltip-128:after {
    margin-top: -28px;
}

.tooltip-256:after {
    margin-top: -28px;
}

img.badge-64 {
    width: 64px;
    height: 64px;
}

img.badge-128 {
    width: 128px;
    height: 128px;
}
